#include <stdint.h>

#include "date_time.h"

// Gets the day number for a date.
uint32_t date_time_get_day_number(uint16_t year, uint8_t month, uint8_t day) {
	month = (month + 9) % 12;
	year = year - month / 10;
	return 365UL * year + year / 4 - year / 100 + year / 400 + (month * 306 + 5) / 10 + day - 1;	
}

// Gets the day number for a date relative to the CP/M date epoch (31/12/1977).
uint16_t date_time_get_day_number_from_epoch(uint16_t year, uint8_t month, uint8_t day) {
	return date_time_get_day_number(year, month, day) - CPM_EPOCH;
}

// Gets the date from a day number.
void date_time_get_date_from_day_number(uint32_t day_number, uint16_t* year, uint8_t* month, uint8_t* day) {
	*year = ((uint64_t)10000 * day_number + 14780) / (uint64_t)3652425;
	int32_t ddd = day_number - (365L * *year + *year / 4L - *year / 100 + *year / 400L);
	if (ddd < 0) {
		*year = *year - 1;
		ddd = day_number - (365L * *year + *year / 4L - *year / 100L + *year / 400L);
	}
	uint32_t mi = (100UL * ddd + 52UL) / 3060UL;
	*month = (mi + 2) % 12 + 1;
	*year = *year + (mi + 2) / 12;
	*day = ddd - (mi * 306 + 5) / 10 + 1;
}

// Converts a two-digit year to four-digit year.
uint16_t date_time_expand_year(uint8_t year) {
	uint16_t expanded_year = ((DATE_TIME_YEAR_CUTOFF / 100) * 100) + year;
	if (year < (DATE_TIME_YEAR_CUTOFF % 100)) expanded_year += 100;
	return expanded_year;
}