#ifndef DATE_TIME_H
#define DATE_TIME_H

#include <stdint.h>

// 31/12/1977 expressed as a day number.
#define CPM_EPOCH 722389UL

#define DATE_TIME_YEAR_CUTOFF (1930)

// Represents a date and time as used by CP/M.
struct date_time_t {
	// Date (days since the CP/M epoch).
	uint16_t date;
	// Hours (BCD).
	uint8_t hour;
	// Minutes (BCD).
	uint8_t minute;
	// Seconds (BCD).
	uint8_t second;
};

// Gets the day number for a date.
uint32_t date_time_get_day_number(uint16_t year, uint8_t month, uint8_t day);

// Gets the day number for a date relative to the CP/M date epoch (31/12/1977).
uint16_t date_time_get_day_number_from_epoch(uint16_t year, uint8_t month, uint8_t day);

// Gets the date from a day number.
void date_time_get_date_from_day_number(uint32_t day_number, uint16_t* year, uint8_t* month, uint8_t* day);

// Sets the time using the RTC.
extern uint8_t date_time_set_rtc(struct date_time_t* date_time);

// Gets the time from the RTC.
extern uint8_t date_time_get_rtc(struct date_time_t* date_time);

// Checks if the RTC time is valid; if not, the clock is reset when booting.
extern uint8_t date_time_rtc_is_valid(void);

// Converts a two-digit year to four-digit year.
uint16_t date_time_expand_year(uint8_t year);

#endif