#include <avr/io.h>

#include "mcp23s08.h"
#include "spi.h"

// Initialise access to the MCP23S08.
void mcp23s08_init(void) {
	spi_init();
	// Enable hardware addressing.
	mcp23s08_write(0b00, MCP23S08_IOCON, _BV(MCP23S08_HAEN));
	// Tristate outputs.
	mcp23s08_write(0b00, MCP23S08_IODIR, 0xFF);
	mcp23s08_write(0b01, MCP23S08_IODIR, 0xFF);
	mcp23s08_write(0b10, MCP23S08_IODIR, 0xFF);
	mcp23s08_write(0b11, MCP23S08_IODIR, 0xFF);
}

// Write a value to an MCP23S08 register.
void mcp23s08_write(uint8_t address, enum mcp23s08_register_t reg, uint8_t value) {
	MCP23S08_PORT &= (uint8_t)~_BV(MCP23S08_SS_BIT);
	spi_transmit(0b01000000 | (address << 1));
	spi_transmit(reg);
	spi_transmit(value);
	MCP23S08_PORT |= _BV(MCP23S08_SS_BIT);
}

// Read a value from a MCP23S08 register.
uint8_t mcp23s08_read(uint8_t address, enum mcp23s08_register_t reg) {
	MCP23S08_PORT &= (uint8_t)~_BV(MCP23S08_SS_BIT);
	spi_transmit(0b01000001 | (address << 1));
	spi_transmit(reg);
	uint8_t value = spi_transmit(0x00);
	MCP23S08_PORT |= _BV(MCP23S08_SS_BIT);
	return value;
}