#ifndef SD_H
#define SD_H

#define SD_SS_PORT PORTB
#define SD_SS_PIN PINB
#define SD_SS_DDR DDRB

#define SD_SS 3

// Commands supported by SD cards in SPI mode.
enum sd_command_t {
	SD_CMD_GO_IDLE_STATE = 0,
	SD_CMD_SEND_OP_COND = 1,
	SD_CMD_SWITCH_FUNC = 6,
	SD_CMD_SEND_IF_COND = 8,
	SD_CMD_SEND_CSD = 9,
	SD_CMD_SEND_CID = 10,
	SD_CMD_STOP_TRANSMISSION = 12,
	SD_CMD_SEND_STATUS = 13,
	SD_CMD_SET_BLOCKLEN = 16,
	SD_CMD_READ_SINGLE_BLOCK = 17,
	SD_CMD_READ_MULTIPLE_BLOCK = 18,
	SD_CMD_WRITE_BLOCK = 24,
	SD_CMD_WRITE_MULTIPLE_BLOCK = 25,
	SD_CMD_PROGRAM_CSD = 27,
	SD_CMD_SET_WRITE_PROT = 28,
	SD_CMD_CLR_WRITE_PROT = 29,
	SD_CMD_SEND_WRITE_PROT = 30,
	SD_CMD_ERASE_WR_BLK_START_ADDR = 32,
	SD_CMD_ERASE_WR_BLK_END_ADDR = 33,
	SD_CMD_ERASE = 38,
	SD_CMD_LOCK_UNLOCK = 42,
	SD_CMD_APP_CMD = 55,
	SD_CMD_GEN_CMD = 56,
	SD_CMD_READ_OCR = 58,
	SD_CMD_CRC_ON_OFF = 59,
};

// Generic errors reported by the SD card routines.
enum sd_error_t {
	SD_ERROR_OK,
	SD_ERROR_UNSPECIFIED,
	SD_ERROR_TIMEOUT,	
	SD_ERROR_CRC,
};

struct sd_csd_t {
	uint8_t CRC_1 : 1;
	uint8_t CRC : 7;
	uint8_t reserved_5 : 2;
	uint8_t FILE_FORMAT : 2;
	uint8_t TMP_WRITE_PROTECT : 1;
	uint8_t PERM_WRITE_PROTECT : 1;
	uint8_t COPY : 1;
	uint8_t FILE_FORMAT_GRP : 1;	
	uint8_t reserved_4 : 5;
	uint8_t WRITE_BL_PARTIAL : 1;
	uint8_t WRITE_BL_LEN : 4;
	uint8_t R2W_FACTOR : 3;
	uint8_t reserved_3 : 2;
	uint8_t WP_GRP_ENABLE : 1;
	uint8_t WP_GRP_SIZE : 7;
	uint8_t SECTOR_SIZE : 7;
	uint8_t ERASE_BLK_EN : 1;
	uint8_t C_SIZE_MULT : 3;
	uint8_t VDD_W_CURR_MAX : 3;
	uint8_t VDD_W_CURR_MIN : 3;
	uint8_t VDD_R_CURR_MAX : 3;
	uint8_t VDD_R_CURR_MIN : 3;
	uint16_t C_SIZE : 12;
	uint8_t reserved_2 : 2;
	uint8_t DSR_IMP : 1;
	uint8_t READ_BLK_MISALIGN : 1;
	uint8_t WRITE_BLK_MISALIGN : 1;
	uint8_t READ_BL_PARTIAL : 1;
	uint8_t READ_BL_LEN : 4;
	uint16_t CCC : 12;
	uint8_t TRAN_SPEED : 8;
	uint8_t NSAC : 8;
	uint8_t TAAC : 8;
	uint8_t reserved_1 : 6;
	uint8_t CSD_STRUCTURE : 2;
};

// Temporary storage for a complete SD card block (512 bytes).
extern uint8_t sd_block[512];

// Resets and initialises the SD card.
enum sd_error_t sd_reset(void);

// Reads a 512 byte block from the specified address.
enum sd_error_t sd_read_block(uint32_t address, uint8_t* buffer);

// Writes a 512 byte block to the specified address.
enum sd_error_t sd_write_block(uint32_t address, uint8_t* buffer);

// Reads a 128 byte record from the specified address.
enum sd_error_t sd_read_record(uint32_t address, uint8_t* buffer);

// Reads the CSD from the card.
enum sd_error_t sd_read_csd(struct sd_csd_t* csd);

#endif