#ifndef SERIAL_H
#define SERIAL_H

#define SERIAL_RX_BUFFER_SIZE (64)
#define SERIAL_RX_BUFFER_FULL_THRESHOLD (SERIAL_RX_BUFFER_SIZE/2)

enum serial_handshake_t {
	SERIAL_HANDSHAKE_NONE,
	SERIAL_HANDSHAKE_XON_XOFF,
	SERIAL_HANDSHAKE_RTS_CTS,
};

// Defines the current handshake mode.
extern enum serial_handshake_t serial_handshake;

// Initialises the serial port at a particular baud rate.
void serial_init(uint16_t baud_rate);

// Determines whether the serial transmitter is ready to send data or not.
uint8_t serial_transmitter_is_ready(void);

// Transmits a byte out of the serial port.
void serial_transmit(uint8_t data);

// Amount of data in the receive buffer.
volatile extern uint8_t serial_rx_buffer_count;

// Flag set when serial data has been received.
volatile extern uint8_t serial_received_flag;

// Read a byte from the serial port's receive buffer.
uint8_t serial_receive();

#define SERIAL_CTS_PORT PORTD
#define SERIAL_CTS_PIN PORTD
#define SERIAL_CTS_DDR DDRD
#define SERIAL_CTS 2

#define SERIAL_RTS_PORT PORTD
#define SERIAL_RTS_PIN PORTD
#define SERIAL_RTS_DDR DDRD
#define SERIAL_RTS 3

#endif